@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-categories-master-list-array.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -22,16 +22,21 @@
  */
 #include <config.h>
 
-#include "e-categories-master-list-array.h"
-
-#include <gal/util/e-i18n.h>
-#include <gal/util/e-xml-utils.h>
-#include <gnome-xml/tree.h>
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
-#include "e-unicode.h"
+#include <stdlib.h>
 #include <string.h>
 
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
+
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
+#include "gal/util/e-util-private.h"
+#include "gal/util/e-xml-utils.h"
+
+#include "e-categories-master-list-array.h"
+#include "e-unicode.h"
+
 #define PARENT_TYPE e_categories_master_list_get_type ()
 
 #define d(x)
@@ -58,34 +63,34 @@ struct _ECategoriesMasterListArrayPriv {
 
 static ECategoriesMasterListClass *parent_class;
 
-struct {
+static struct {
 	gchar *category;
 	gchar *icon;
 } builtin_categories[] = {
-	{ N_("Birthday"),         GAL_IMAGESDIR "/category_birthday_16.png" },
-	{ N_("Business"),         GAL_IMAGESDIR "/category_business_16.png" },
+	{ N_("Birthday"),         "category_birthday_16.png" },
+	{ N_("Business"),         "category_business_16.png" },
 	{ N_("Competition"),      NULL },
-	{ N_("Favorites"),        GAL_IMAGESDIR "/category_favorites_16.png" },
-	{ N_("Gifts"),            GAL_IMAGESDIR "/category_gifts_16.png" },
-	{ N_("Goals/Objectives"), GAL_IMAGESDIR "/category_goals_16.png" },
-	{ N_("Holiday"),          GAL_IMAGESDIR "/category_holiday_16.png" },
-	{ N_("Holiday Cards"),    GAL_IMAGESDIR "/category_holiday-cards_16.png" },
-	{ N_("Hot Contacts"),     GAL_IMAGESDIR "/category_hot-contacts_16.png" },
-	{ N_("Ideas"),            GAL_IMAGESDIR "/category_ideas_16.png" },
-	{ N_("International"),    GAL_IMAGESDIR "/category_international_16.png" },
-	{ N_("Key Customer"),     GAL_IMAGESDIR "/category_key-customer_16.png" },
-	{ N_("Miscellaneous"),    GAL_IMAGESDIR "/category_miscellaneous_16.png" },
-	{ N_("Personal"),         GAL_IMAGESDIR "/category_personal_16.png" },
-	{ N_("Phone Calls"),      GAL_IMAGESDIR "/category_phonecalls_16.png" },
-	{ N_("Status"),           GAL_IMAGESDIR "/category_status_16.png" },
-	{ N_("Strategies"),       GAL_IMAGESDIR "/category_strategies_16.png" },
-	{ N_("Suppliers"),        GAL_IMAGESDIR "/category_suppliers_16.png" },
-	{ N_("Time & Expenses"),  GAL_IMAGESDIR "/category_time-and-expenses_16.png" },
+	{ N_("Favorites"),        "category_favorites_16.png" },
+	{ N_("Gifts"),            "category_gifts_16.png" },
+	{ N_("Goals/Objectives"), "category_goals_16.png" },
+	{ N_("Holiday"),          "category_holiday_16.png" },
+	{ N_("Holiday Cards"),    "category_holiday-cards_16.png" },
+	{ N_("Hot Contacts"),     "category_hot-contacts_16.png" },
+	{ N_("Ideas"),            "category_ideas_16.png" },
+	{ N_("International"),    "category_international_16.png" },
+	{ N_("Key Customer"),     "category_key-customer_16.png" },
+	{ N_("Miscellaneous"),    "category_miscellaneous_16.png" },
+	{ N_("Personal"),         "category_personal_16.png" },
+	{ N_("Phone Calls"),      "category_phonecalls_16.png" },
+	{ N_("Status"),           "category_status_16.png" },
+	{ N_("Strategies"),       "category_strategies_16.png" },
+	{ N_("Suppliers"),        "category_suppliers_16.png" },
+	{ N_("Time & Expenses"),  "category_time-and-expenses_16.png" },
 	{ N_("VIP"),              NULL },
 	{ N_("Waiting"),          NULL },
 };
 
-#define BUILTIN_CATEGORY_COUNT (sizeof(builtin_categories) / sizeof(builtin_categories[0]))
+#define BUILTIN_CATEGORY_COUNT G_N_ELEMENTS (builtin_categories)
 
 static void
 ecmlai_free (ECategoriesMasterListArrayItem *ecmlai)
@@ -301,9 +306,17 @@ ecmla_default (ECategoriesMasterListArray *ecmla)
 	ecmla->priv->categories = g_new (ECategoriesMasterListArrayItem *, BUILTIN_CATEGORY_COUNT);
 
 	for (i = 0; i < BUILTIN_CATEGORY_COUNT; i++) {
-		char *category = e_utf8_from_locale_string(_(builtin_categories[i].category));
-		ecmla->priv->categories[i] = ecmlai_new (category, builtin_categories[i].icon, NULL);
-		g_free (category);
+		char *category = _(builtin_categories[i].category);
+		char *icon_filename = NULL;
+
+		if (builtin_categories[i].icon != NULL)
+			icon_filename =
+				g_build_filename (GAL_IMAGESDIR,
+						  builtin_categories[i].icon,
+						  NULL);
+		ecmla->priv->categories[i] =
+			ecmlai_new (category, icon_filename, NULL);
+		g_free (icon_filename);
 	}
 }
 
@@ -321,7 +334,7 @@ ecmla_reset (ECategoriesMasterList *ecml)
 }
 
 static void
-ecmla_destroy (GtkObject *object)
+ecmla_dispose (GObject *object)
 {
 	ECategoriesMasterListArray *ecmla = E_CATEGORIES_MASTER_LIST_ARRAY (object);
 
@@ -329,17 +342,17 @@ ecmla_destroy (GtkObject *object)
 	g_free (ecmla->priv);
 	ecmla->priv = NULL;
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
 
 static void
-ecmla_class_init (GtkObjectClass *object_class)
+ecmla_class_init (GObjectClass *object_class)
 {
 	ECategoriesMasterListClass *ecml_class = E_CATEGORIES_MASTER_LIST_CLASS(object_class);
 
-	parent_class          = gtk_type_class (PARENT_TYPE);
+	parent_class          = g_type_class_ref (PARENT_TYPE);
 
 	ecml_class->count     = ecmla_count    ;
 	ecml_class->nth       = ecmla_nth      ;
@@ -351,7 +364,7 @@ ecmla_class_init (GtkObjectClass *object_class)
 
 	ecml_class->reset     = ecmla_reset    ;
 
-	object_class->destroy = ecmla_destroy  ;
+	object_class->dispose = ecmla_dispose  ;
 }
 
 static void
@@ -367,33 +380,17 @@ ecmla_init (ECategoriesMasterListArray *ecmla)
 	ecmla_default (ecmla);
 }
 
-guint
-e_categories_master_list_array_get_type (void)
-{
-	static guint type = 0;
-	
-	if (!type) {
-		GtkTypeInfo info = {
-			"ECategoriesMasterListArray",
-			sizeof (ECategoriesMasterListArray),
-			sizeof (ECategoriesMasterListArrayClass),
-			(GtkClassInitFunc) ecmla_class_init,
-			(GtkObjectInitFunc) ecmla_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		type = gtk_type_unique (PARENT_TYPE, &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE (e_categories_master_list_array,
+	     "ECategoriesMasterListArray",
+	     ECategoriesMasterListArray,
+	     ecmla_class_init,
+	     ecmla_init,
+	     PARENT_TYPE)
 
 ECategoriesMasterList *
 e_categories_master_list_array_new       (void)
 {
-	return E_CATEGORIES_MASTER_LIST (gtk_type_new (e_categories_master_list_array_get_type ()));
+	return E_CATEGORIES_MASTER_LIST (g_object_new (E_CATEGORIES_MASTER_LIST_ARRAY_TYPE, NULL));
 }
 
 void
